--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
Combuctor_LibFilter

Author: Daphne Pfister <tiiroib02@sneakemail.com>

Adds LibFilter support to Combuctor

License

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

-- Need to decide which SVN server revision number I will use and fix this
-- probably will got with svn.wowace.com's version for the long run
local MAJOR, MINOR = "Combuctor_LibFilter-1.0", 303 -- tonumber(("$Revisions$):match("(%d+)"))
local lib = LibStub:NewLibrary(MAJOR,MINOR)
if not lib then return end

local LibFilter = LibStub("LibFilter-1.0")
local CombuctorSet
local CombuctorFE

lib.known_rule_types = lib.known_rule_types or {}
lib.known_filters = lib.known_filters or {}
lib.known_presets = lib.known_presets or {}
lib.known_parent_sets = lib.known_parent_sets or {}
lib.hooks = lib.hooks or {}
lib.origCalls = lib.origCalls or {}
lib.events = lib.events or {}

local known_rule_types = lib.known_rule_types
local known_filters = lib.known_filters
local known_presets = lib.known_presets
local known_parent_sets = lib.known_parent_sets
local hooks = lib.hooks
local origCalls = lib.origCalls
local events = lib.events
local adapt_log = {}
local do_upgrade = nil
local strjoin = strjoin

local ITEM_TYPE = LibFilter.FilterType.ITEM
local SLOT_TYPE = LibFilter.FilterType.SLOT

local function Print(msg)
	local info = ChatTypeInfo["SYSTEM"]
	DEFAULT_CHAT_FRAME:AddMessage(msg, info.r, info.g, info.b, info.id)
end

local function Print_AdaptLog()
	for filtername, status in pairs(adapt_log) do
		Print(("Combuctor_LibFilter: %s |cFFFFFFFF%s|r"):format(status,filtername))
	end
end

SLASH_COMBUCTORLIBFILTER1 = "/combuctor_libfilter"
SLASH_COMBUCTORLIBFILTER2 = "/clf"
SlashCmdList["COMBUCTORLIBFILTER"] = Print_AdaptLog

function lib:TestFilter(filter,type,preset,link)
	if type == SLOT_TYPE then
		local bag,slot = lib.bag,lib.slot
		if bag and slot then
			local baglink = GetContainerItemLink(bag,slot)
			if (not link or baglink == link) then
				--Print(("Testing slot: %s:%d:%d %s"):format(type,bag,slot,link or "????"))
				if filter.TestIfMatchSplit then
					return filter:TestIfMatchSplit(preset,type,bag,slot) and true or false
				else
					return filter:TestIfMatch(preset,strjoin(":",type,bag,slot)) and true or false
				end
			else
				--Print(("Can not test slot: %s:%d:%d %s"):format(type,bag,slot,baglink or "????"))
			end
		end
		type = ITEM_TYPE
	end
	if type == ITEM_TYPE then
		--Print(("Testing: %s preset: %s link: %s"):format(filter.name,preset,link or "nothing"))
		return filter:TestIfMatch(preset,link) and true or false
	end
	return false
end

function lib:UpdatePresets(filtername,filter,type)
	known_presets[filtername] = known_presets[filtername] or {}
	local presets = known_presets[filtername]

	local filterdesc = filter.name or filtername

	-- Mark all existing presets as unseen incase the need to be removed
	for _,preset in pairs(presets) do
		preset.seen = nil
		preset.updated = nil
		if not known_parent_sets[filtername] then
			-- Upgrade from experimental version
			CombuctorSet:Unregister(preset.desc,preset.parent_desc)
			preset.set = nil
		end
		preset.old_desc = preset.desc
	end

	-- Populate presets
	local any_seen
	local best_icon = filter.icon
	for name in filter:IteratePresets(type) do
		any_seen = true
		local long_desc,icon,desc
		if filter.DescribeSettings then
			long_desc,icon,desc = filter:DescribeSettings(name)
		end
		desc = desc or long_desc or preset
		if desc == "" then
			desc = filter.name or filter.key
		end
		if name == "" then
			if not best_icon then best_icon = icon end
		end
		--Print(("Combuctor_LibFilter: Saw |cFFFFFFFF%s:%s|r"):format(desc,parent or "root"))
		presets[name] = presets[name] or {}
		local preset = presets[name]
		preset.seen = true
		if preset.desc and desc ~= preset.desc then
			if preset.set then
				CombuctorSet:Unregister(preset.desc,filterdesc)
				preset.set = nil
			end
		end
		preset.desc = desc
		preset.name = name
		preset.icon = icon
	end

	-- Create parent set
	if any_seen then
		if not known_parent_sets[filtername] then
			CombuctorSet:Register(filterdesc,best_icon,function() return true end)
			known_parent_sets[filtername] = true
			adapt_log[filtername] = "Registered"
		end
	elseif known_parent_sets[filtername] then
		CombuctorSet:Unregister(filterdesc)
		known_parent_sets[filtername] = nil
		adapt_log[filtername] = "Unregistered"
	end

	-- Update sets
	for name,preset in pairs(presets) do
		local pname = name
		local filter = filter
		if preset.seen then
			if not preset.set then
				CombuctorSet:RegisterSubSet(preset.desc,filterdesc,preset.icon,
																		function (player, bagType, name, link)
																			return lib:TestFilter(filter,type,pname,link)
																		end)
				preset.set = true
			end
		elseif preset.set then
			CombuctorSet:Unregister(preset.desc,filterdesc)
			presets[name] = nil
		end
	end
end

function lib:UpdateFilter(filtername,filter,type)
	-- Kinda hackish
	if not CombuctorFE then
		if Combuctor then
			CombuctorFE = Combuctor:GetModule('FrameEvents')
		end
	end
	if CombuctorFE then
		CombuctorFE:UpdateSets("COMBUCTOR_SET_UPDATE",filter.name or filtername)
	end
end

local function AddFilterRuleType(filtername,filter)
	-- Check to see if filter support presets
	if not filter.IteratePresets then return end

	if known_filters[filtername] and not do_upgrade then return end
	known_filters[filtername] = filter

	local prefered_type = ITEM_TYPE
	if (filter.filtertype == SLOT_TYPE or
			(filter.filtertypes and filter.filtertypes[SLOT_TYPE])) then
		prefered_type = SLOT_TYPE
	end

	filter.RegisterCallback(lib,"PresetUpdate",function() lib:UpdatePresets(filtername,filter,prefered_type) end)
	filter.RegisterCallback(lib,"FilterUpdate",function() lib:UpdateFilter(filtername,filter,prefered_type) end)
	lib:UpdatePresets(filtername,filter,prefered_type)
end

-- Hack to get access to bag and slot numbers for filter
function lib.HasItemHook(orig, ItemFrame, ...)
	lib.bag, lib.slot, lib.link = ...
	local res = orig(ItemFrame, ...)
	lib.bag, lib.slot, lib.link = nil, nil, nil
	return res
end

local function UpdateFilters()
	if not CombuctorSet then
		if Combuctor then
			CombuctorSet = Combuctor:GetModule('Sets')
		end
	end
	if Combuctor and Combuctor.ItemFrame then
		-- Majorily hackish -- allow slot filters to work
		local ItemFrame = Combuctor.ItemFrame
		if not hooks["ItemFrame:HasItem"] then
			hooks["ItemFrame:HasItem"] = true
			local orig = ItemFrame.HasItem
			origCalls["ItemFrame:HasItem"] = orig
			ItemFrame.HasItem = function(...)
														return lib.HasItemHook(orig,...)
													end
		end
	end

	if CombuctorSet then
		for filtername,filter in LibFilter:IterateFilters(SLOT_TYPE,ITEM_TYPE) do
			AddFilterRuleType(filtername,filter)
		end
		do_upgrade = nil
	end
end

lib.myFrame = lib.myFrame or CreateFrame("Frame","Combuctor_LibFilter_Frame")
local myFrame = lib.myFrame
myFrame:SetScript("OnEvent",function (self, event, ...) events[event](...) end)

function events.UPDATE()
	UpdateFilters()
	myFrame:SetScript("OnUpdate",nil)
end

local function update_next_frame()
	myFrame:SetScript("OnUpdate",function (self, ...) events.UPDATE(...) end)
end

function events.ADDON_LOADED()
	update_next_frame()
end

function events.PLAYER_LOGIN()
	update_next_frame()
	myFrame:RegisterEvent("ADDON_LOADED")
end

do_upgrade = true
if IsLoggedIn() then
	update_next_frame()
	myFrame:RegisterEvent("ADDON_LOADED")
else
	myFrame:RegisterEvent("PLAYER_LOGIN")
end
