--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
LibFilter_Professions

Author: Daphne Pfister <tiiroib02@sneakemail.com>

License

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local pt = LibStub("LibPeriodicTable-3.1")

local MAJOR, MINOR = "LibFilter-item-Professions", 303 -- tonumber(("$Revisions$):match("(%d+)"))
local filter = LibStub:NewLibrary(MAJOR, MINOR)
if not filter then return end

filter.filtertype = "item"
filter.key = "Professions"

filter.callbacks = filter.callbacks or LibStub("CallbackHandler-1.0"):New(filter)
local callbacks = filter.callbacks

-- Avoid Global Lookups
local _G                        = getfenv(0)
local BOOKTYPE_SPELL            = _G.BOOKTYPE_SPELL
local ChatTypeInfo              = _G.ChatTypeInfo
local CreateFrame               = _G.CreateFrame
local GetLocale                 = _G.GetLocale
local GetProfessionInfo         = _G.GetProfessionInfo
local GetProfessions            = _G.GetProfessions
local GetSpellBookItemName      = _G.GetSpellBookItemName
local GetSpellInfo              = _G.GetSpellInfo
local GetSpellTabInfo           = _G.GetSpellTabInfo
local LibStub                   = _G.LibStub
local ipairs                    = _G.ipairs
local next                      = _G.next
local pairs                     = _G.pairs
local print                     = _G.print
local select                    = _G.select
local string                    = _G.string
local strjoin                   = _G.strjoin
local table                     = _G.table
local tonumber                  = _G.tonumber
local type                      = _G.type
local strmatch                  = string.match
local tinsert                   = table.insert
local tsort                     = table.sort

-- No need for a full blown locale library here
local L
do
	local LL = {
		enUS = {
			["Professions"] = true, -- Filter name
			["Type"] = true, -- Filter type
			["Items Craftable by Known Professions"] = true,
			["Items Craftable by Known Primary Professions"] = true,
			["Items Craftable by Unknown Professions"] = true,
			["Items Craftable by Unknown Primary Professions"] = true,
			["Crafted Items"] = true,
			["Items Gathered by Known Professions"] = true,
			["Items Gathered by Known Primary Professions"] = true,
			["Items Gathered by Unknown Professions"] = true,
			["Items Gathered by Unknown Primary Professions"] = true,
			["Gathered Items"] = true,
			["Items Used by Known Professions"] = true,
			["Items Used by Known Primary Professions"] = true,
			["Items Used by Unknown Professions"] = true,
			["Items Used by Unknown Primary Professions"] = true,
			["Trade Materials"] = true,
			["Tools Used by Known Professions"] = true,
			["Tools Used by Known Primary Professions"] = true,
			["Tools Used by Unknown Professions"] = true,
			["Tools Used by Unknown Primary Professions"] = true,
			["Professional Tools"] = true,
			["Recipes for Known Professions"] = true,
			["Recipes for Known Primary Professions"] = true,
			["Recipes for Unknown Professions"] = true,
			["Recipes for Unknown Primary Professions"] = true,
			["Recipes"] = true,
			["Filter by items related to Known / Unknown Professions"] = true,
			["Known Professions"] = true,
			["Match only for professions known to the character"] = true,
			["Unknown Professions"] = true,
			["Match if used only by professions unknown to the character"] = true,
			["Primary and Secondary Professions"] = true,
			["Match for both primary and secondary professions"] = true,
			["Primary Professions Only"] = true,
			["Match only for primary professions"] = true,
		},
	}

	local gGameLocale = GetLocale()
	local enL = LL["enUS"]
	L = LL[gGameLocale] or LL["enUS"]

	for vKey,vValue in pairs(enL) do
		if not L[vKey] then L[vKey] = vValue end
	end
	for vKey,vValue in pairs(L) do
		if vValue == true then L[vKey] = vKey end
	end
end

local TESTTYPE_CRAFTED = "Crafted"
local TESTTYPE_GATHER = "Gather"
local TESTTYPE_MAT = "Mat"
local TESTTYPE_TOOL = "Tool"
local TESTTYPE_RECIPE = "Recipe"

-- Professions
local ALCHEMY                                   = (GetSpellInfo(2259))
--cal ALCHEMY_ELIXIR                            = (GetSpellInfo(28677))
--cal ALCHEMY_POTION                            = (GetSpellInfo(28675))
--cal ALCHEMY_TRANSMUTATION                     = (GetSpellInfo(28672))
local ARCHAEOLOGY                               = (GetSpellInfo(78670))
local BLACKSMITHING                             = (GetSpellInfo(2018))
local BLACKSMITHING_ARMORSMITH                  = (GetSpellInfo(9788))
local BLACKSMITHING_WEAPONSMITH                 = (GetSpellInfo(9787))
local BLACKSMITHING_WEAPONSMITH_AXESMITH        = (GetSpellInfo(17041))
local BLACKSMITHING_WEAPONSMITH_HAMMERSMITH     = (GetSpellInfo(17040))
local BLACKSMITHING_WEAPONSMITH_SWORDSMITH      = (GetSpellInfo(17039))
local COOKING                                   = (GetSpellInfo(2550))
local DISENCHANT                                = (GetSpellInfo(13262))
local ENCHANTING                                = (GetSpellInfo(7411))
local ENGINEERING                               = (GetSpellInfo(4036))
local ENGINEERING_GNOMISH                       = (GetSpellInfo(20219))
local ENGINEERING_GOBLIN                        = (GetSpellInfo(20222))
local FIRST_AID                                 = (GetSpellInfo(3273))
local FISHING                                   = (GetSpellInfo(131474))
local HERBALISM                                 = (GetSpellInfo(2383))
local INSCRIPTION                               = (GetSpellInfo(45357))
local JEWELCRAFTING                             = (GetSpellInfo(25229))
local LEATHERWORKING                            = (GetSpellInfo(2108))
local LEATHERWORKING_DRAGONSCALE                = (GetSpellInfo(10656))
local LEATHERWORKING_ELEMENTAL                  = (GetSpellInfo(10658))
local LEATHERWORKING_TRIBAL                     = (GetSpellInfo(10660))
local MILLING                                   = (GetSpellInfo(51005))
local MINING                                    = (GetSpellInfo(2575))
local PICKPOCKETING                             = (GetSpellInfo(921))
--cal POISONS                                   = (GetSpellInfo(2842))
local PROSPECTING                               = (GetSpellInfo(31252))
local SKINNING                                  = (GetSpellInfo(8613))
local SMELTING                                  = (GetSpellInfo(2656))
local TAILORING                                 = (GetSpellInfo(3908))
local TAILORING_MOONCLOTH                       = (GetSpellInfo(26798))
local TAILORING_SHADOWEAVE                      = (GetSpellInfo(26801))
local TAILORING_SPELLFIRE                       = (GetSpellInfo(26797))

local ProfessionsToPT3 = {
  [ALCHEMY]                                     = "Alchemy",
--[ALCHEMY_ELIXIR]                              = "Alchemy.Elixir Master",
--[ALCHEMY_POTION]                              = "Alchemy.Potion Master",
--[ALCHEMY_TRANSMUTATION]                       = "Alchemy.Transmutation Master",
--[ARCHAELOGY]                                  = "Archaelogy",
  [BLACKSMITHING]                               = "Blacksmithing.Basic",
  [BLACKSMITHING_ARMORSMITH]                    = "Blacksmithing.Armorsmith",
  [BLACKSMITHING_WEAPONSMITH]                   = "Blacksmithing.Weaponsmith.Basic",
  [BLACKSMITHING_WEAPONSMITH_AXESMITH]          = "Blacksmithing.Weaponsmith.Axesmith",
  [BLACKSMITHING_WEAPONSMITH_HAMMERSMITH]       = "Blacksmithing.Weaponsmith.Hammersmith",
  [BLACKSMITHING_WEAPONSMITH_SWORDSMITH]        = "Blacksmithing.Weaponsmith.Swordsmith",
  [COOKING]                                     = "Cooking",
  [DISENCHANT]                                  = "Disenchant",
  [ENCHANTING]                                  = "Enchanting",
  [ENGINEERING]                                 = "Engineering.Basic",
  [ENGINEERING_GNOMISH]                         = "Engineering.Gnomish",
  [ENGINEERING_GOBLIN]                          = "Engineering.Goblin",
  [FIRST_AID]                                   = "First Aid",
  [FISHING]                                     = "Fishing",
  [HERBALISM]                                   = "Herbalism",
  [INSCRIPTION]                                 = "Inscription",
  [JEWELCRAFTING]                               = "Jewelcrafting",
  [LEATHERWORKING]                              = "Leatherworking.Basic",
  [LEATHERWORKING_DRAGONSCALE]                  = "Leatherworking.Dragonscale",
  [LEATHERWORKING_ELEMENTAL]                    = "Leatherworking.Elemental",
  [LEATHERWORKING_TRIBAL]                       = "Leatherworking.Tribal",
  [MILLING]                                     = "Milling",
  [MINING]                                      = "Mining",
  [PICKPOCKETING]                               = "Pickpocketing",
--[POISONS]                                     = "Poisons",
  [PROSPECTING]                                 = "Prospecting",
  [SKINNING]                                    = "Skinning",
  [SMELTING]                                    = "Smelting",
  [TAILORING]                                   = "Tailoring.Basic",
  [TAILORING_MOONCLOTH]                         = "Tailoring.Mooncloth",
  [TAILORING_SHADOWEAVE]                        = "Tailoring.Shadoweave",
  [TAILORING_SPELLFIRE]                         = "Tailoring.Spellfire",
}

local Specializations = {
--[ALCHEMY_ELIXIR] = ALCHEMY,
--[ALCHEMY_POTION] = ALCHEMY,
--[ALCHEMY_TRANSMUTATION] = ALCHEMY,
  [BLACKSMITHING_ARMORSMITH] = BLACKSMITHING,
	[BLACKSMITHING_WEAPONSMITH] = BLACKSMITHING,
	[BLACKSMITHING_WEAPONSMITH_AXESMITH] = BLACKSMITHING,
	[BLACKSMITHING_WEAPONSMITH_HAMMERSMITH] = BLACKSMITHING,
	[BLACKSMITHING_WEAPONSMITH_SWORDSMITH] = BLACKSMITHING,
	[ENGINEERING_GNOMISH] = ENGINEERING,
	[ENGINEERING_GOBLIN] = ENGINEERING,
	[LEATHERWORKING_DRAGONSCALE] = LEATHERWORKING,
	[LEATHERWORKING_ELEMENTAL] = LEATHERWORKING,
	[LEATHERWORKING_TRIBAL] = LEATHERWORKING,
  [TAILORING_MOONCLOTH] = TAILORING,
  [TAILORING_SHADOWEAVE] = TAILORING,
  [TAILORING_SPELLFIRE] = TAILORING,
}

local SecondaryProfessions = {
--[ARCHAELOGY] = "Archaelogy",
	[COOKING] = "Cooking",
	[FIRST_AID] = "First Aid",
	[FISHING] = "Fishing",
	[PICKPOCKETING]	 = "Pickpocketing",
}

local TypeLogic = {
	[TESTTYPE_CRAFTED] = {
		Key = TESTTYPE_CRAFTED,
		Name = "Items Craftable by %s %s",
		Description = L["Crafted Items"],
		Everything = "Tradeskill.Crafted",
	},
	[TESTTYPE_GATHER] = {
		Key = TESTTYPE_GATHER,
		Name = "Items Gathered by %s %s",
		Description = L["Gathered Items"],
		Everything = "Tradeskill.Gather",
	},
	[TESTTYPE_MAT] = {
		Key = TESTTYPE_MAT,
		Name = "Items Used by %s %s",
		Description = L["Trade Materials"],
		Everything = "Tradeskill.Mat.ByProfession",
	},
	[TESTTYPE_TOOL] = {
		Key = TESTTYPE_TOOL,
		Name = "Tools Used by %s %s",
		Description = L["Professional Tools"],
		Everything = "Tradeskill.Tool",
	},
	[TESTTYPE_RECIPE] = {
		Key = TESTTYPE_RECIPE,
		Name = "Recipes for %s %s",
		Description = L["Recipes"],
		Everything = "Tradeskill.Recipe", -- Might be better to use GetItemInfo for recipe
	},
}

filter.known_skills = filter.known_skills or {}
local known_skills = filter.known_skills

local function update()
	callbacks:Fire("FilterUpdate")
end

local function UpdateSkills()
	local old_known = {}
	for k in pairs(known_skills) do
		old_known[k] = true
		known_skills[k] = nil
	end
  local known_profs = { GetProfessions() }
  for profIndex in pairs(known_profs) do
		local prof = known_profs[ profIndex ]
    if prof then
			local skillName = GetProfessionInfo(prof)
      if skillName then
				--print("Skill ", skillName)
				known_skills[skillName] = 1
			end
    end
  end

  -- Scan for specializations
	local lastId = select(4, GetSpellTabInfo(1))
	for spellId = 1, lastId do
		local skillName = GetSpellBookItemName(spellId,BOOKTYPE_SPELL)
		if skillName and Specializations[skillName] then
			--print("Skill ", skillName)
			known_skills[skillName] = 1
		end
	end

	for k in pairs(known_skills) do
		if not old_known[k] then
			-- We learned a new skill
			update()
			return
		else
			old_known[k] = nil
		end
	end

	if next(old_known) then
		-- We forgot an old one
		update()
		return
	end
end

filter.events = filter.events or {}
filter.myFrame = filter.myFrame or CreateFrame("Frame","LibFilter_Professions_Frame")
local events = filter.events
local myFrame = filter.myFrame
myFrame:SetScript("OnEvent",function (self, event, ...) events[event](...) end)

events.PLAYER_LOGIN = UpdateSkills
events.SKILL_LINES_CHANGED = UpdateSkills

filter.author = "Nayala"
filter.name = L["Professions"]
filter.description = L["Filter by items related to Known / Unknown Professions"]

function filter:NewSettings(settings)
	return {
		test = settings and settings.test or TESTTYPE_MAT,
		other = settings and settings.other or false,
		primary = settings and settings.primary or false,
		alt = settings and settings.alt or nil, -- Future
	}
end

function filter:FreeSettings(settings)
end

function filter:DescribeSettings(settings)
	local test = settings and settings.test or TESTTYPE_MAT
	local typelogic = TypeLogic[test] or TypeLogic[TESTTYPE_MAT]

	return L[typelogic.Name:format(settings and settings.other and "Unknown" or "Known",
																 settings and settings.primary and "Primary Professions" or "Professions")]
end
-- /print LibStub("LibFilter-item-Professions"):SettingsOptions({}).args
function filter:SettingsOptions(pSettings,pTable,pMethod)
	local vCallback = update

	if pTable then
		if type(pTable) == "string" then
			vCallback = function() pTable[pMethod](pTable,pSettings) end
		else
			vCallback = function() pTable(pSettings) end
		end
	end

	if not pSettings then return end

	local test = pSettings.test
	if not test or not TypeLogic[test] then
		test = TESTTYPE_MAT
		pSettings.test = test
	end

	local args = {
		{
			order = 1,
			type = "toggle",
			name = L["Known Professions"],
			desc = L["Match only for professions known to the character"],
			get = function() return not pSettings.other end,
			set = function(v)
							if pSettings.other then
								pSettings.other = false
								vCallback()
							end
						end,
		},
		{
			order = 2,
			type = "toggle",
			name = L["Unknown Professions"],
			desc = L["Match if used only by professions unknown to the character"],
			get = function() return pSettings.other end,
			set = function(v)
							if not pSettings.other then
								pSettings.other = true
								vCallback()
							end
						end,
		},
		{
			order = 3,
			type = "toggle",
			name = L["Primary and Secondary Professions"],
			desc = L["Match for both primary and secondary professions"],
			get = function() return not pSettings.primary end,
			set = function(v)
							if pSettings.primary then
								pSettings.primary = false
								vCallback()
							end
						end,
		},
		{
			order = 4,
			type = "toggle",
			name = L["Primary Professions Only"],
			desc = L["Match only for primary professions"],
			get = function() return pSettings.primary end,
			set = function(v)
							if not pSettings.primary then
								pSettings.primary = true
								vCallback()
							end
						end,
		},
		{
			order = 5,
			type = "header",
			name = L["Type"],
		},
	}
	local testnames = {}
	for test in pairs(TypeLogic) do
		tinsert(testnames,test)
	end
	tsort(testnames)
	local order = #args
	for _,test in ipairs(testnames) do
		local testlogic = TypeLogic[test]
		order = order + 1
		args[order] = {
			type = "toggle",
			order = order,
			name = testlogic.Description,
			desc = testlogic.Description,
			get = function() return pSettings.test == test end,
			set = function(v)
							if pSettings.test ~= test then
								pSettings.test = test
								vCallback()
							end
						end,
		}
	end
	return { type = "group", desc = "LibFilter_Professions options", args = args }
end

local function populate_skills(typelogic)
	--print("Populating: "..typelogic.Key)
	local base = typelogic.Everything
	local skills = {}
	for skill, pt3name in pairs(ProfessionsToPT3) do
		local name = strjoin(".",base,pt3name)
		local shortname = strmatch(name,"(.*)%.Basic")
		if pt:GetSetString(name) then
			skills[skill] = name
			--print(("Skill: %s |cFFFFFFFF%s|r"):format(skill, name))
		elseif shortname and pt:GetSetString(shortname) then
			skills[skill] = shortname
			--print(("Skill: %s |cFFFFFFFF%s|r"):format(skill, shortname))
		end
	end
	typelogic.Skills = skills
end

function filter:TestIfMatch(settings,link)
	if not link then return end
	local test = settings and settings.test or TESTTYPE_MAT
	local typelogic = TypeLogic[test] or TypeLogic[TESTTYPE_MAT]
	local primary = settings and settings.primary
	local other = settings and settings.other

	local itemid = link:match("item:(%d+)")
	itemid = tonumber(itemid)

	if not pt:ItemInSet(itemid,typelogic.Everything) then return end

	if not typelogic.Skills then
		populate_skills(typelogic)
	end

	local match = nil
	for skill,set in pairs(typelogic.Skills) do
		if primary and SecondaryProfessions[skill] then
			-- Skip this skill
		elseif other then
			-- Match if the item is match by any unknown skill
			-- and not known by any of ours
			if pt:ItemInSet(itemid,set) then
				if known_skills[skill] then
					return nil
				else
					match = true
				end
			end
		else
			-- Match if matched by any known skill
			if known_skills[skill] then
				if pt:ItemInSet(itemid,set) then
					return true
				end
			end
		end

	end
	return match
end

myFrame:SetScript("OnEvent",function (self, event, ...) events[event](...) end)
myFrame:RegisterEvent("PLAYER_LOGIN")
myFrame:RegisterEvent("SKILL_LINES_CHANGED")
if next(known_skills) then UpdateSkills() end
